import React from 'react'
import PropTypes from 'prop-types'
import cn from 'classnames/bind'
import Icon from '../Icons'
import LoadingSpinner, { SPINNER_TYPES } from '../LoadingSpinner'
import { useConsoleWarning } from '../../utils/warnings'
import { useLocalizeAll } from '../../utils/localize'
import styles from './button.less'
import { defaultEventLabel } from '../../constants/dataAnalytics'
import getDataEventContext from '../../utils/analytics'

const cx = cn.bind(styles)

export const buttonTypes = {
  /** Dark grey with white text do not invert, use secondary */
  primary: styles.primary,
  /** White with dark grey text can not be inverted */
  secondary: styles.secondary,
  /** Blue with white text */
  info: styles.info,
  /** Blue Green with white text this will replace info when we update the accessible blue color everywhere */
  infoNew: styles.infoNew,
  /** Orange with white text */
  danger: styles.danger,
  /** Green with white text */
  success: styles.success,
  /** Light Gray with white text */
  complete: styles.complete,
  /** Pink with white text */
  editable: styles.editable,
  /** Purple with white text replace editable when we update the color everywhere */
  editableNew: styles.editableNew,
  /** Blue Text and underline no border */
  link: styles.link,
  /** Red Text and underline no border */
  dangerLink: styles.dangerLink,
  /** Blue Green Text and underline no border will replace link when we update the accessible blue color everywhere */
  infoLink: styles.infoLink,
}

export const mobileBreakPoints = {
  medium: styles.hiddenMd,
  small: styles.hiddenSm,
  xSmall: styles.hiddenXs,
}

const Button = ({
  ariaLabel,
  asLink,
  buttonClass,
  buttonIcon,
  buttonIconRight,
  buttonIconSize,
  buttonText,
  children,
  disabled,
  eventContext,
  eventLabel,
  extraButtonProps,
  hideLabelAtBreakPoint,
  inverted,
  isLoading,
  isSubmit,
  onClick,
  shouldLocalize,
  type,
}) => {
  // Throw accessibility warning if no aria label in provided for icon only buttons
  useConsoleWarning({
    message: 'ACCESSIBILITY WARNING: Aria label is required if no button text is used',
    showMessage: buttonIcon && !buttonText && !ariaLabel,
  })

  const ButtonElement = asLink ? 'a' : 'button'
  const hasIcon = buttonIcon || buttonIconRight
  const buttonStyles = cx(buttonClass, type, {
    [styles.disabled]: disabled || isLoading,
    [styles.inverted]: inverted,
    [styles.loading]: isLoading,
    [styles.leftIcon]: buttonIcon,
    [styles.rightIcon]: buttonIconRight,
  })
  const buttonContentClass = cx(styles.buttonContent, {
    [hideLabelAtBreakPoint]: hideLabelAtBreakPoint && hasIcon,
  })
  const localizedButtonText = useLocalizeAll`${buttonText}`
  const telemetryContext = getDataEventContext(eventContext)

  return (
    <ButtonElement
      className={buttonStyles}
      disabled={disabled || isLoading}
      onClick={onClick}
      aria-label={ariaLabel || buttonText}
      data-event-label={eventLabel || buttonText || defaultEventLabel}
      type={isSubmit ? 'submit' : 'button'}
      role={asLink ? 'link' : 'button'}
      {...telemetryContext}
      {...extraButtonProps}
    >
      <span className={styles.spinner} aria-hidden={!isLoading}>
        <LoadingSpinner size={SPINNER_TYPES.xs} />
      </span>
      <span className={buttonContentClass}>
        {children || (
          <>
            {buttonIcon && <Icon icon={buttonIcon} size={buttonIconSize} />}
            <span className={styles.buttonText}>
              {shouldLocalize ? localizedButtonText : buttonText}
            </span>
            {buttonIconRight && <Icon icon={buttonIconRight} size={buttonIconSize} />}
          </>
        )}
      </span>
    </ButtonElement>
  )
}
Button.defaultProps = {
  ariaLabel: null,
  asLink: false,
  buttonClass: null,
  buttonIcon: null,
  buttonIconRight: null,
  buttonIconSize: 13,
  buttonText: '',
  children: null,
  disabled: false,
  eventContext: null,
  eventLabel: null,
  extraButtonProps: null,
  hideLabelAtBreakPoint: null,
  inverted: false,
  isLoading: false,
  isSubmit: false,
  onClick: () => null,
  shouldLocalize: true,
  type: buttonTypes.primary,
}

Button.propTypes = {
  /** Aria label required if only using icon or if button text is vague */
  ariaLabel: PropTypes.string,
  /** Is this Button functioning as a link? */
  asLink: PropTypes.bool,
  /** Additional class if needed */
  buttonClass: PropTypes.string,
  /** Icon to add to the left of the button */
  buttonIcon: PropTypes.func,
  /** Icon to the right of button text if needed */
  buttonIconRight: PropTypes.func,
  /** Change Icon size, this should only be used in special cases */
  buttonIconSize: PropTypes.number,
  /** Text for the button */
  buttonText: PropTypes.string,
  /** This prop should only be used in rare cases if you need a more custom button */
  children: PropTypes.node,
  /** State at which to disable onClick */
  disabled: PropTypes.bool,
  /** Context object used for telemetry analysis */
  eventContext: PropTypes.shape({}),
  /** Custom optional attribute used for data analytics tracking */
  eventLabel: PropTypes.string,
  /** Any additional attributes you want to give to the button */
  extraButtonProps: PropTypes.shape({}),
  /** Do not display the label in mobile views by screen size */
  hideLabelAtBreakPoint: PropTypes.oneOf(Object.values(mobileBreakPoints)),
  /** Do you want to invert the colors of the button */
  inverted: PropTypes.bool,
  /** State at which to show loading spinner */
  isLoading: PropTypes.bool,
  /** Is this button being used a a form submit button */
  isSubmit: PropTypes.bool,
  /** Action to take onClick */
  onClick: PropTypes.func,
  /** Do you want to localize the button text */
  shouldLocalize: PropTypes.bool,
  /** Type of button, import button types for a complete list, primary is default */
  type: PropTypes.oneOf(Object.values(buttonTypes)),
}

export default Button
