import React, { useEffect, useRef } from 'react'
import cx from 'classnames'
import PropTypes from 'prop-types'
import Label from '../Label'
import { useConsoleWarning } from '../../utils/warnings'
import styles from './checkBox.less'

const Checkbox = ({
  bold,
  className,
  disabled,
  error,
  id,
  indeterminate,
  inline,
  isChecked,
  isFormGroupHeading,
  label,
  labelClass,
  large,
  name,
  onBlur,
  onChange,
  required,
  shouldLocalize,
  title,
  value,
  ...rest
}) => {
  const checkboxRef = useRef()
  useEffect(() => {
    if (indeterminate !== null) checkboxRef.current.indeterminate = indeterminate
  }, [checkboxRef, indeterminate])
  useConsoleWarning({
    message:
      'ACCESSIBILITY WARNING: Checkboxes without associated labels are not accessible and present general usability issues. See https://uxmovement.com/forms/why-your-checkboxes-need-to-have-label-tags/ for reference.',
    showMessage: !label,
  })
  return (
    <div
      className={className || styles.checkBoxContainer}
      style={{ display: inline && 'inline-block' }}
    >
      <div className={cx(styles.checkBox, { [styles.large]: large })}>
        <input
          checked={isChecked}
          className={cx(styles.prettyCheckBox)}
          disabled={disabled}
          aria-checked={isChecked}
          aria-describedby={error ? `${id}-error` : null}
          aria-label={label}
          id={id}
          name={name}
          onBlur={onBlur}
          onChange={onChange}
          readOnly={!onChange}
          ref={checkboxRef}
          title={title}
          type="checkbox"
          value={value}
          {...rest}
        />
        {label && (
          <Label
            forId={id}
            label={label}
            className={cx(labelClass, {
              [styles.checkBoxLabel]: !labelClass,
              [styles.bold]: bold,
            })}
            inline
            formGroupHeading={isFormGroupHeading}
            required={required}
            shouldLocalize={shouldLocalize}
            title={title}
          />
        )}
      </div>
    </div>
  )
}

Checkbox.defaultProps = {
  bold: false,
  className: '',
  disabled: false,
  error: undefined,
  indeterminate: null,
  inline: false,
  isChecked: undefined,
  isFormGroupHeading: false,
  label: '',
  labelClass: null,
  large: false,
  name: undefined,
  onBlur: undefined,
  onChange: undefined,
  required: false,
  shouldLocalize: false,
  title: undefined,
  value: undefined,
}

Checkbox.propTypes = {
  /** The label on the input. */
  label: PropTypes.string,
  /** Class passed to label */
  labelClass: PropTypes.string,
  /** Is the label a form group heading */
  isFormGroupHeading: PropTypes.bool,
  /** The value of the input. */
  value: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.number,
    PropTypes.bool,
    PropTypes.arrayOf(PropTypes.any),
  ]),
  /** Name of input. */
  name: PropTypes.string,
  /** Is it checked? */
  isChecked: PropTypes.bool,
  /** Display inline. */
  inline: PropTypes.bool,
  /** Called when changed. */
  onChange: PropTypes.func,
  /** The id of the input. */
  id: PropTypes.string.isRequired,
  /** Is there a little red *? */
  required: PropTypes.bool,
  /** Is the label bold */
  bold: PropTypes.bool,
  /** String or function passed to ErrorBox */
  error: PropTypes.oneOfType([PropTypes.func, PropTypes.string, PropTypes.bool]),
  /** Is this checkable? */
  disabled: PropTypes.bool,
  /** Custom className */
  className: PropTypes.string,
  /** Called on blurring field. */
  onBlur: PropTypes.func,
  /** Is the checkbox large */
  large: PropTypes.bool,
  /** Should the label be localized? */
  shouldLocalize: PropTypes.bool,
  title: PropTypes.string,
  /** Show indeterminate state */
  indeterminate: PropTypes.bool,
}

export default Checkbox
