import 'react-dates/initialize'
import 'react-dates/lib/css/_datepicker.css'
import cx from 'classnames'
import PropTypes from 'prop-types'
import React, { useState } from 'react'
import { DateRangePicker as AirBnbDateRangePicker } from 'react-dates'
import Icon, { icons } from '../Icons'
import {
  convertToMomentObject,
  formatDate,
  isAfter,
  isBefore,
  isValidDate,
  now,
} from '../../utils/date'
import { useLocalize } from '../../utils/localize'

import styles from './dateRangePicker.less'

const DateRangePicker = ({ id, minDate, maxDate, startDateValue, endDateValue, onChange }) => {
  const [start, setStart] = useState(startDateValue)
  const [end, setEnd] = useState(endDateValue)
  const [focused, setFocused] = useState(null)
  const translation = {
    from: useLocalize`From`,
    to: useLocalize`To`,
  }
  const startDateInputId = `${id}_start_date`
  const endDateInputId = `${id}_end_date`
  const startInput = document.getElementById(startDateInputId)
  const endInput = document.getElementById(endDateInputId)

  const extraProps = {
    isOutsideRange: () => false,
  }

  const validMinDate = minDate && isValidDate(formatDate(minDate)) ? minDate : null
  const validMaxDate = maxDate && isValidDate(formatDate(maxDate)) ? maxDate : null

  if (validMinDate || validMaxDate) {
    extraProps.isOutsideRange = day => {
      return isBefore(day, validMinDate, 'day') || isAfter(day, validMaxDate, 'day')
    }
  }

  const handleDatesChange = ({ startDate, endDate }) => {
    setStart(startDate)
    setEnd(endDate)
    const formattedStart = startDate ? formatDate(startDate) : null
    const formattedEnd = endDate ? formatDate(endDate) : null
    onChange({ startDate: formattedStart, endDate: formattedEnd })
  }

  const removeInvalidInputValues = () => {
    // React Dates persists the input value if it is outside of range
    // but does not send the value to the onchange function
    // This resets the value to an empty string on close when that happens.
    if (!end && endInput) endInput.value = ''
    if (!start && startInput) startInput.value = ''
  }

  const handleFocusChange = focusedInput => {
    setFocused(focusedInput)
  }

  const datePickerClass = cx(styles.wrapper, {
    [styles.focusedBorder]: focused,
  })

  return (
    <div className={datePickerClass}>
      <AirBnbDateRangePicker
        customInputIcon={<Icon size={15} icon={icons.CALENDAR} />}
        customArrowIcon={<Icon size={15} icon={icons.NEXT} />}
        startDate={start ? convertToMomentObject(start) : null}
        startDateId={startDateInputId}
        startDatePlaceholderText={translation.from}
        endDate={end ? convertToMomentObject(end) : null}
        endDateId={endDateInputId}
        endDatePlaceholderText={translation.to}
        onDatesChange={({ startDate, endDate }) => handleDatesChange({ startDate, endDate })}
        focusedInput={focused}
        onFocusChange={handleFocusChange}
        onClose={removeInvalidInputValues}
        minDate={minDate || null}
        {...extraProps}
        small
      />
    </div>
  )
}

DateRangePicker.defaultProps = {
  id: 'date_picker',
  startDateValue: null,
  endDateValue: null,
  maxDate: now(),
  minDate: null,
}

DateRangePicker.propTypes = {
  /**  Function that returns an object { startDate, endDate } formatted MM/DD/YYYY */
  onChange: PropTypes.func.isRequired,
  /** optional id for start and end date inputs gets resolves to ${id}_start_date ${id}_end_date */
  id: PropTypes.string,
  /** Date string */
  startDateValue: PropTypes.string,
  /** Date string */
  endDateValue: PropTypes.string,
  /** Date string */
  maxDate: PropTypes.string,
  /** Date string */
  minDate: PropTypes.string,
}

export default DateRangePicker
