import React, { useState, useEffect, useRef } from 'react'
import PropTypes from 'prop-types'
import cx from 'classnames'
import Icon, { icons } from '../Icons'
import styles from './dropdown.less'
import { detectScrollBarClick } from '../../utils/dom'
import { useLocalizeAll } from '../../utils/localize'

const DropdownHooks = ({
  className,
  label,
  labelComponent,
  children,
  role,
  disabled,
  hideCarret,
  openLeft,
}) => {
  const dropdownRef = useRef(null)
  const menuRef = useRef(null)
  const [isOpen, setIsOpen] = useState(false)
  const [cursor, setCursor] = useState(0)
  const labelText = useLocalizeAll`${label}`

  useEffect(() => {
    const menuContainer = menuRef.current
    const menuButton = dropdownRef.current
    const options = menuRef.current
      ? [...menuContainer.childNodes].filter(node => !node.disabled)
      : []
    if (options.length > 0) {
      options[cursor].focus()
    }
    const toggleDropdown = () => {
      if (isOpen) {
        menuButton.focus()
      }
      setIsOpen(!isOpen)
    }
    const handleOutsideClick = event => {
      const clickedScrollBar = detectScrollBarClick(event)
      if (dropdownRef.current && !dropdownRef.current.contains(event.target) && !clickedScrollBar) {
        toggleDropdown()
      }
    }
    const handleKeydown = event => {
      const size = options.length
      if (event.key === 'ArrowDown') {
        event.preventDefault()
        setCursor(cursor === size - 1 ? 0 : cursor + 1)
      } else if (event.key === 'ArrowUp') {
        event.preventDefault()
        setCursor(cursor === 0 ? size - 1 : cursor - 1)
      }
      if (event.key === 'Escape') {
        event.preventDefault()
        toggleDropdown()
        menuButton.focus()
      }
      if (event.key === 'Tab') {
        event.preventDefault()
        setIsOpen(false)
      }
    }
    if (isOpen) {
      window.addEventListener('click', handleOutsideClick)
      window.addEventListener('keydown', handleKeydown)
    } else {
      window.removeEventListener('click', handleOutsideClick)
    }
    return () => {
      window.removeEventListener('click', handleOutsideClick)
      window.removeEventListener('keydown', handleKeydown)
    }
  }, [isOpen, cursor])

  const buttonClass = cx(styles.buttonReset, {
    [styles.disabled]: disabled,
    [className]: className,
    [styles.dropdownButton]: !className && !labelComponent,
  })

  const dropdownMenuClass = cx(styles.dropdownMenu, {
    [styles.openLeft]: openLeft,
  })

  return (
    <div className={styles.dropdownContainer}>
      <button
        className={buttonClass}
        ref={dropdownRef}
        tabIndex="0"
        disabled={disabled}
        onClick={() => {
          setIsOpen(!isOpen)
        }}
        type="button"
        aria-expanded={isOpen}
        aria-haspopup="true"
      >
        {label && <span aria-label={label}>{labelText}</span>}
        {labelComponent && labelComponent}
        {!hideCarret && <Icon icon={isOpen ? icons.DROPDOWN_UP : icons.DROPDOWN_DOWN} size={16} />}
      </button>
      {isOpen && (
        <div className={dropdownMenuClass} role={role} ref={menuRef}>
          {children}
        </div>
      )}
    </div>
  )
}

DropdownHooks.defaultProps = {
  className: null,
  label: null,
  labelComponent: null,
  role: 'menu',
  disabled: false,
  hideCarret: false,
  openLeft: false,
}

DropdownHooks.propTypes = {
  /** Contents of the dropdown. Make sure to use role="menuitem" for elements. Use <a> tags for a list of links */
  children: PropTypes.node.isRequired,
  /** Additional classname if needed  */
  className: PropTypes.string,
  /** Dropdown label */
  label: PropTypes.string,
  /** Custom dropdown label component */
  labelComponent: PropTypes.node,
  /** Custom dropdownMenu role */
  role: PropTypes.string,
  /** Is the dropdown disabled? */
  disabled: PropTypes.bool,
  /** Hide the dropdown carret?  */
  hideCarret: PropTypes.bool,
  /** Open the modal left */
  openLeft: PropTypes.bool,
}

const Dropdown = props => {
  return <DropdownHooks {...props} />
}

Dropdown.defaultProps = DropdownHooks.defaultProps
Dropdown.propTypes = DropdownHooks.propTypes

export default Dropdown
