import React from 'react'
import { Link } from 'react-router-dom'
import PropTypes from 'prop-types'
import cx from 'classnames'
import { useLocalizeAll } from '../../utils/localize'
import styles from '../../theme/linkStyles.less'
import { defaultEventLabel } from '../../constants/dataAnalytics'
import getDataEventContext from '../../utils/analytics'

const InternalLink = ({
  ariaLabel,
  children,
  className,
  eventContext,
  eventLabel,
  linkText,
  shouldLocalize,
  to,
  useReactRouter,
  ...props
}) => {
  const classes = cx(styles.accessibleLink, {
    [className]: className,
    [styles.link]: !className,
  })
  const localizedText = useLocalizeAll`${linkText}`
  const telemetryContext = getDataEventContext(eventContext)

  if (useReactRouter)
    return (
      <Link
        to={to}
        aria-label={ariaLabel || linkText}
        data-event-label={eventLabel || linkText || defaultEventLabel}
        className={classes}
        {...telemetryContext}
        {...props}
      >
        {shouldLocalize ? localizedText : linkText}
        {children}
      </Link>
    )
  return (
    <a
      href={to}
      aria-label={ariaLabel || linkText}
      data-event-label={eventLabel || linkText || defaultEventLabel}
      className={classes}
      {...telemetryContext}
      {...props}
    >
      {shouldLocalize ? localizedText : linkText}
      {children}
    </a>
  )
}

export default InternalLink

InternalLink.defaultProps = {
  ariaLabel: '',
  children: null,
  className: null,
  eventContext: null,
  eventLabel: null,
  linkText: null,
  shouldLocalize: true,
  to: '',
  useReactRouter: false,
}
InternalLink.propTypes = {
  /** Required if there is no link text or if the linktext doesn't obviously describe the link */
  ariaLabel: PropTypes.string,
  /** Optionally render custom children in link */
  children: PropTypes.node,
  /** Override default link styles */
  className: PropTypes.string,
  /** Context object used for telemetry analysis */
  eventContext: PropTypes.shape({}),
  /** Custom optional attribute used for data analytics tracking */
  eventLabel: PropTypes.string,
  /** Will localize text by default */
  linkText: PropTypes.string,
  /** Defaults to TRUE do not translate user generated text */
  shouldLocalize: PropTypes.bool,
  /** Href for <a> tag */
  to: PropTypes.string,
  /** Are we linking back to a react page? */
  useReactRouter: PropTypes.bool,
}
