import React from 'react'
import PropTypes from 'prop-types'
import classNames from 'classnames'
import Cleave from 'cleave.js/react'
import styles from './maskInput.less'

export const maskConfiguration = Object.freeze({
  EIN: {
    blocks: [2, 7],
    delimiter: '-',
    numericOnly: true,
  },
  SSN_ITIN: {
    blocks: [3, 2, 4],
    delimiter: '-',
    numericOnly: true,
  },
  CURRENCY_USD: {
    numeral: true,
    numeralDecimalMark: '.',
    numeralDecimalScale: 2,
    numeralThousandsGroupStyle: 'thousand',
  },
})

const MaskInput = ({
  containerClass,
  describedBy,
  disabled,
  hasError,
  id,
  inputClass,
  maskProps,
  onChange,
  placeholder,
  required,
  type,
  value,
  ...rest
}) => {
  const inputProps = {
    'aria-required': required,
    'aria-describedby': describedBy?.toString().replace(',', ' '),
    className: classNames(
      styles.inputClass,
      {
        [styles.inputError]: !disabled && hasError,
        [styles.disabled]: disabled,
      },
      inputClass
    ),
    disabled,
    id,
    onChange,
    options: maskProps,
    placeholder,
    required,
    type,
    value,
    ...rest,
  }

  return (
    <div className={classNames(styles.inputContainer, containerClass)}>
      <Cleave {...inputProps} />
    </div>
  )
}

MaskInput.defaultProps = {
  containerClass: undefined,
  describedBy: undefined,
  disabled: false,
  hasError: false,
  id: null,
  inputClass: undefined,
  maskProps: { delimiter: '' },
  onChange: null,
  placeholder: '',
  required: false,
  type: 'text',
  value: undefined,
}

MaskInput.propTypes = {
  /** aria text displayed under input */
  describedBy: PropTypes.oneOfType([PropTypes.string, PropTypes.arrayOf(PropTypes.string)]),
  /** Class passed to container div */
  containerClass: PropTypes.string,
  /** Can you type in the input? */
  disabled: PropTypes.bool,
  /** Error message displayed under input */
  hasError: PropTypes.bool,
  /** The input of the id */
  id: PropTypes.string,
  /** Class passed to input div */
  inputClass: PropTypes.string,
  /** Mask Props: https://github.com/nosir/cleave.js/blob/master/doc/options.md */
  maskProps: PropTypes.shape({
    /** It detects credit card type dynamically and automatically by checking card IIN. */
    creditCard: PropTypes.bool,
    /** Expand use of 19-digit PANs for supported credit card. */
    creditCardStrictMode: PropTypes.bool,
    /** The unique String argument type is the type of the detected credit, which can be:
     amex mastercard visa diners discover jcb dankort instapayment uatp mir unionPay */
    onCreditCardTypeChanged: PropTypes.func,
    /** A Boolean value indicates if this is a date input field. Enable to trigger date shortcut mode. */
    date: PropTypes.bool,
    /** An Array value indicates the date pattern. */
    datePattern: PropTypes.arrayOf(PropTypes.string),
    /** An date String value indicates the min date boundary.
     The date string format follows as ISO 8601 date format YYYY-MM-DD */
    dateMin: PropTypes.string,
    /** An date String value indicates the max date boundary.
     The date string format follows as ISO 8601 date format YYYY-MM-DD */
    dateMax: PropTypes.string,
    /** A Boolean value indicates if this is a time input field. Enable to trigger time shortcut mode. */
    time: PropTypes.bool,
    /** An Array value indicates the time pattern. */
    timePattern: PropTypes.arrayOf(PropTypes.string),
    /** A String value indicates time format
     Default value '24' Military time format */
    timeFormat: PropTypes.string,
    /** A Boolean value indicates if this is a numeral input field. Enable to trigger numeral shortcut mode. */
    numeral: PropTypes.bool,
    /** A String value indicates the thousands separator grouping style. */
    numeralThousandsGroupStyle: PropTypes.oneOf(['thousand', 'lakh', 'wan', 'none']),
    /** A value indicates the numeral integer scale. */
    numeralIntegerScale: PropTypes.number,
    /** A value indicates the numeral decimal scale. */
    numeralDecimalScale: PropTypes.number,
    /** A value indicates the numeral decimal mark. */
    numeralDecimalMark: PropTypes.string,
    /** A value indicates if it only allows positive numeral value */
    numeralPositiveOnly: PropTypes.bool,
    /** A value indicates if the sign of the numeral should appear before the prefix. */
    signBeforePrefix: PropTypes.bool,
    /** A value makes prefix should be appear after the numeral. */
    tailPrefix: PropTypes.bool,
    /** A value indicates if zeroes appearing at the beginning of the number should be stripped out. This also prevents a number like "100,000" to disappear if the leading "1" is deleted. */
    stripLeadingZeroes: PropTypes.bool,
    /** A value indicates the delimiter to use in formatting. */
    delimiter: PropTypes.string,
    /** A value indicates the multiple delimiters to use in formatting. */
    delimiters: PropTypes.arrayOf(PropTypes.string),
    /** A value indicates the prepend string. It can't be removed or changed in the input field. */
    prefix: PropTypes.string,
    /** A value indicates if it only allows numeric letters (0-9). */
    numericOnly: PropTypes.bool,
    /** An Array value indicates the groups to format the input value. It will insert delimiters in between these groups. */
    blocks: PropTypes.arrayOf(PropTypes.number),
  }),
  /** The placeholder on the input */
  placeholder: PropTypes.string,
  /** Called on Change */
  onChange: PropTypes.func,
  /** Is the input *required? */
  required: PropTypes.bool,
  /** The input type attribute value */
  type: PropTypes.string,
  /** The value of the input */
  value: PropTypes.string,
}

export default MaskInput
