import React, { useState, useEffect, useRef } from 'react'
import cx from 'classnames'
import PropTypes from 'prop-types'
import Icon, { icons } from '../Icons'
import styles from './modal.less'
import { handleOutsideClick, useStopDocumentScroll } from '../../utils/dom'
import { useLocalize } from '../../utils/localize'
import FocusTrap from '../../utils/FocusTrap'

export const useModal = () => {
  const [isShowing, setIsShowing] = useState(false)
  const toggleModal = () => setIsShowing(!isShowing)
  return {
    isShowing,
    toggleModal,
  }
}

const ModalHooks = ({
  onClose,
  icon,
  warn,
  style = {},
  showBackdrop,
  children,
  fullscreen,
  className,
  modalBodyClassName,
  enableCloseButton,
  focusFirst,
  focusLast,
  modalClassName,
}) => {
  const modalBodyRef = useRef(null)
  useStopDocumentScroll()
  useEffect(() => {
    const handleModalOutsideClick = event => handleOutsideClick(event, modalBodyRef, onClose)
    const handleEscapeButton = event => {
      if (event.key === 'Escape') onClose()
    }
    if (modalBodyRef.current)
      setTimeout(() => window.addEventListener('mousedown', handleModalOutsideClick), 200)
    window.addEventListener('keydown', handleEscapeButton)
    return () => {
      setTimeout(() => window.removeEventListener('mousedown', handleModalOutsideClick), 200)
      window.removeEventListener('keydown', handleEscapeButton)
    }
  }, [onClose, modalBodyRef, children])

  return (
    <FocusTrap focusFirst={focusFirst} focusLast={focusLast}>
      <div
        className={cx(modalClassName, {
          [styles.backdrop]: showBackdrop,
          [styles.noBackdrop]: !showBackdrop,
          [styles.fullscreen]: fullscreen,
        })}
        role="dialog"
        aria-modal="true"
        aria-describedby="dialog-description"
      >
        <div
          className={`${styles.modalStyle} ${className || ''}`}
          style={style}
          role="presentation"
          ref={modalBodyRef}
          onClick={event => event.stopPropagation()}
        >
          {icon && (
            <div className={styles.modalIcon}>
              <Icon className={styles.icon} icon={icon} />
            </div>
          )}
          {warn && (
            <div className={styles.modalIcon}>
              <span>!</span>
            </div>
          )}
          {enableCloseButton && (
            <button className={styles.closeButton} type="button" onClick={onClose} aria-hidden>
              <Icon className={styles.closeIcon} icon={icons.CLOSE} />
              <span className="sr-only">{useLocalize`Close`}</span>
            </button>
          )}
          <div id="dialog-description" className={modalBodyClassName || styles.modalBody}>
            {children}
          </div>
        </div>
      </div>
    </FocusTrap>
  )
}

ModalHooks.defaultProps = {
  children: [],
  className: null,
  modalBodyClassName: null,
  enableCloseButton: false,
  icon: null,
  showBackdrop: true,
  fullscreen: false,
  style: null,
  warn: false,
  focusFirst: false,
  focusLast: false,
  modalClassName: '',
}

ModalHooks.propTypes = {
  /** Should contain logic to dismiss the Modal */
  onClose: PropTypes.func.isRequired,
  // Optionals:
  children: PropTypes.node,
  /** Classname for full screen */
  className: PropTypes.string,
  /** Classname for the body of the modal */
  modalBodyClassName: PropTypes.string,
  /** Do you want an optional close X in the top right corner */
  enableCloseButton: PropTypes.bool,
  /** Should this modal cover the full screen */
  fullscreen: PropTypes.bool,
  /** Should the backdrop be visible */
  showBackdrop: PropTypes.bool,
  /** any optional styles you need */
  style: PropTypes.shape(),
  /** Icon to show in orange circle */
  icon: PropTypes.func,
  /** Show a ! instead of an icon */
  warn: PropTypes.bool,
  /** Should the first element be focused on mount */
  focusFirst: PropTypes.bool,
  /** Should the last element be focused on mount */
  focusLast: PropTypes.bool,
  /** Classname for full screen */
  modalClassName: PropTypes.string,
}

const Modal = props => {
  return <ModalHooks {...props} />
}

export default Modal
