import React, { forwardRef } from 'react'
import PropTypes from 'prop-types'
import classNames from 'classnames'
import TextareaAutosize from 'react-autosize-textarea'
import { useLocalizeAll } from '../../utils/localize'
import styles from './textInput.less'

const TextInput = forwardRef(
  (
    {
      value,
      onChange,
      containerClass,
      disabled,
      inline,
      showErrorStyle,
      error,
      description,
      required,
      placeholder,
      id,
      type,
      inputGroupFirst,
      inputGroupLast,
      inputWidth,
      isMultiLine,
      rows,
      willHavePresence,
      showPresenceInline,
      ...rest
    },
    ref
  ) => {
    const inputProps = {
      id,
      value,
      onChange,
      disabled,
      required,
      type,
      'aria-required': required,
      'aria-describedby': description && `${id}-description`,
      style: { width: inputWidth },
      className: classNames(styles.inputClass, {
        [styles.inputError]: (!disabled && error) || showErrorStyle,
        [styles.disabled]: disabled,
        [styles.inputGroupFirst]: inputGroupFirst,
        [styles.inputGroupLast]: inputGroupLast,
      }),
      ...rest,
    }

    const containerClasses = classNames(styles.inputContainer, {
      [styles.inline]: inline || showPresenceInline,
      [containerClass]: containerClass,
    })
    const inputPlaceholder = useLocalizeAll`${placeholder}`
    const InputComponent = isMultiLine ? TextareaAutosize : 'input'

    return (
      <div className={containerClasses}>
        <InputComponent rows={rows} ref={ref} placeholder={inputPlaceholder} {...inputProps} />
      </div>
    )
  }
)

TextInput.defaultProps = {
  onChange: null,
  value: undefined,
  required: false,
  containerClass: undefined,
  disabled: false,
  inline: false,
  error: undefined,
  description: undefined,
  id: null,
  type: 'text',
  placeholder: '',
  onKeyDown: undefined,
  inputGroupFirst: false,
  inputGroupLast: false,
  inputWidth: null,
  showErrorStyle: false,
  isMultiLine: false,
  rows: null,
  willHavePresence: false,
  showPresenceInline: false,
}

TextInput.propTypes = {
  /** The placeholder on the input */
  placeholder: PropTypes.string,
  /** Called on Change */
  onChange: PropTypes.func,
  /** The value of the input */
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.shape({}), PropTypes.number]),
  /** Is the input *required? */
  required: PropTypes.bool,
  /** Class passed to container div */
  containerClass: PropTypes.string,
  /** Can you type in the input? */
  disabled: PropTypes.bool,
  /** Is display inline-block? */
  inline: PropTypes.bool,
  /** Error message displayed under input */
  error: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.bool]),
  /** Descriptive text displayed under input */
  description: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  /** The input of the id */
  id: PropTypes.string,
  /** The input type attribute value */
  type: PropTypes.string,
  /** Function called when input is in focus and specified key is pressed */
  onKeyDown: PropTypes.func,
  /** Is this part of a form group and the first child */
  inputGroupFirst: PropTypes.bool,
  /** Is this part of a form group and the last child */
  inputGroupLast: PropTypes.bool,
  /** Custom input width */
  inputWidth: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  /** Apply error styling */
  showErrorStyle: PropTypes.bool,
  /** Is this a Text Area or Text Input */
  isMultiLine: PropTypes.bool,
  /** Number of rows for textarea */
  rows: PropTypes.number,
  /** Do you expect to show presence */
  willHavePresence: PropTypes.bool,
  showPresenceInline: PropTypes.bool,
}

export default TextInput
