import React, { useState, useEffect, useRef } from 'react'
import PropTypes from 'prop-types'
/** A utility to trap focus in a given component.
 * It will return focus the the previously focused acitve element on unmount.
 * This is useful when building accessible modals */
const FocusTrap = ({ children, focusFirst, focusLast }) => {
  const [previouslyFocusedElement] = useState(document?.activeElement)
  const focusTrapArea = useRef(null)

  useEffect(() => {
    let allInputs = focusTrapArea?.current?.querySelectorAll(
      'a[href]:not([disabled]), button:not([disabled]), textarea:not([disabled]), input[type="text"]:not([disabled]), input[type="radio"]:not([disabled]), input[type="checkbox"]:not([disabled]), input[type="number"]:not([disabled]), select:not([disabled]), [role="menu"]'
    )
    allInputs = Array.from(allInputs)
    const inputs = allInputs.filter(input => !input.disabled)
    const hasFocusableAreas = inputs.length > 0
    const firstInput = hasFocusableAreas ? inputs[0] : null
    const lastInput = hasFocusableAreas ? inputs[inputs.length - 1] : null
    const onlyHasOneFocusableArea = inputs.length === 1

    const firstInputListener = event => {
      if (event.key === 'Tab' && event.shiftKey && lastInput) {
        if (onlyHasOneFocusableArea) firstInput.focus()
        event.preventDefault()
        lastInput.focus()
      }
    }

    const lastInputListener = event => {
      if (event.key === 'Tab' && !event.shiftKey && firstInput) {
        if (onlyHasOneFocusableArea) firstInput.focus()
        event.preventDefault()
        firstInput.focus()
      }
    }

    if (hasFocusableAreas) {
      if (focusFirst || !focusLast) {
        // Focus on first focusable element by default
        firstInput.focus()
      }
      if (focusLast) {
        lastInput.focus()
      }
      firstInput.addEventListener('keydown', firstInputListener)
      lastInput.addEventListener('keydown', lastInputListener)
    }

    const unmount = () => {
      if (hasFocusableAreas) {
        firstInput.removeEventListener('keydown', firstInputListener)
        lastInput.removeEventListener('keydown', lastInputListener)
      }
      previouslyFocusedElement.focus()
    }

    return () => unmount()
  }, [previouslyFocusedElement, focusTrapArea, focusFirst, focusLast])
  return <div ref={focusTrapArea}>{children}</div>
}

FocusTrap.defaultProps = {
  focusFirst: false,
  focusLast: false,
}

FocusTrap.propTypes = {
  /** Wrapped component you want to trap focus in */
  children: PropTypes.node.isRequired,
  /** Should the first element be focused on mount */
  focusFirst: PropTypes.bool,
  /** Should the last element be focused on mount */
  focusLast: PropTypes.bool,
}

export default FocusTrap
