import { useLayoutEffect } from 'react'
/**
 * Returns a function that cancels a DOM event passed as its argument, then
 * invokes the given action.
 *
 * This is sugar to avoid having to explicitly handle event cancellation in
 * event handlers on </a> elements, thus keeping the handlers clean.
 * @param {function} action
 */
export function cancelEventThen(action) {
  return e => {
    e.preventDefault()
    e.stopPropagation()
    action()
  }
}

const inputTags = ['input', 'select', 'button', 'textarea']

/**
 * Returns true if the active element is an input, textarea, select or button.
 */
export function isInputElementActive() {
  const { activeElement } = document
  return activeElement && inputTags.indexOf(activeElement.tagName.toLowerCase()) !== -1
}

/**
 * Prevent scrolling on document body.
 */
export const setDocumentOverFlowHidden = () => {
  document.body.style.overflow = 'hidden'
}

/**
 * Enable scrolling on document body.
 */
export const setDocumentOverFlowScroll = () => {
  document.body.style.overflow = ''
}

/**
 * Set document overflow style.
 * @param {string} style the style of overflow you want to set
 */
export const setDocumentOverFlowStyle = style => {
  document.body.style.overflow = style
}
/**
 * Detect if a user clicks the scroll bar
 * @param {node} event
 */
export const detectScrollBarClick = event => {
  const hasScrollbar = window.innerWidth > document.documentElement.clientWidth
  const clickedScrollbar = hasScrollbar
    ? document.documentElement.clientWidth <= event.clientX
    : document.documentElement.clientWidth - 17 <= event.clientX
  return clickedScrollbar
}

/**
 * Genereic handle outside click function and takes the scrollbar into account
 * @param {node} event - the click event
 * @param {node} ref - the inner ref
 * @param {function} action - what to do when a user clicks outside
 */
export const handleOutsideClick = (event, ref, action) => {
  const clickedScrollBar = detectScrollBarClick(event)
  if (ref.current !== null && !ref.current.contains(event.target) && !clickedScrollBar) {
    action()
  }
}

export const useStopDocumentScroll = () => {
  useLayoutEffect(() => {
    const originalStyle = window.getComputedStyle(document.body).overflow
    setDocumentOverFlowHidden()
    return () => setDocumentOverFlowStyle(originalStyle)
  }, [])
}
